'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function createHoc(hook) {
    return (map) => (component) => {
        const hoc = (props) => {
            const [state, actions] = hook();
            return react.createElement(component, Object.assign({}, props, map(state, actions)));
        };
        hoc.displayName = `WithDakpan(${component.displayName || component.name || 'Component'})`;
        return hoc;
    };
}

function createHook(context) {
    return () => {
        const state = react.useContext(context);
        if (!state) {
            throw Error('Provider not mounted');
        }
        return [
            state.state,
            state.actions
        ];
    };
}

function createProvider(context, actions, initialState) {
    return ({ children, value = initialState }) => {
        const [state, setState] = react.useState(value);
        const mapped = react.useMemo(() => Object.keys(actions).reduce((previous, current) => {
            const execute = (...args) => () => Promise.resolve(actions[current](...args)(state))
                .then((next) => {
                if (next) {
                    setState(next);
                }
            });
            const action = (...args) => execute(...args)();
            action.c = execute;
            return Object.assign({}, previous, { [current]: action });
        }, {}), [actions, state]);
        return react.createElement(context.Provider, {
            value: {
                state,
                actions: mapped
            }
        }, children);
    };
}

function createDakpan(initialState) {
    return (actions) => {
        const context = react.createContext(undefined);
        const hook = createHook(context);
        return [
            createProvider(context, actions, initialState),
            hook,
            createHoc(hook)
        ];
    };
}

exports.createDakpan = createDakpan;
//# sourceMappingURL=index.js.map
